Package: cvcqv
Type: Package
Title: Coefficient of Variation (CV) with Confidence Intervals (CI)
Version: 1.0.3
Date: 2026-05-08
Authors@R: person("Maani", "Beigy", email = "manibeygi@gmail.com",
                  role = c("aut", "cre"))
Maintainer: Maani Beigy <manibeygi@gmail.com>
Description: Provides some easy-to-use functions and classes to calculate
    variability measures such as coefficient of variation with confidence
    intervals provided with all available methods. References are
    'Panichkitkosolkul' (2013) <doi:10.1155/2013/324940>,
    'Altunkaynak' & 'Gamgam' (2018) <doi:10.1080/03610918.2018.1435800>,
    'Albatineh', 'Kibria', Wilcox & 'Zogheib' (2014) <doi:10.1080/02664763.2013.847405>.
Depends: R (>= 3.5.0), dplyr (>= 0.8.0.1)
Imports: R6, SciViews, boot, MBESS
Suggests: testthat, knitr, rmarkdown, covr
VignetteBuilder: knitr
URL: https://github.com/MaaniBeigy/cvcqv
BugReports: https://github.com/MaaniBeigy/cvcqv/issues
License: GPL-3
Encoding: UTF-8
RoxygenNote: 8.0.0
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-05-07 22:19:14 UTC; 20240700
Author: Maani Beigy [aut, cre]
Repository: CRAN
Date/Publication: 2026-05-21 09:30:07 UTC
Built: R 4.5.2; ; 2026-05-25 02:38:25 UTC; unix
