# Generated by roxygen2: do not edit by hand

export(BootCoefQuartVar)
export(BootCoefVar)
export(CoefQuartVar)
export(CoefQuartVarCI)
export(CoefVar)
export(CoefVarCI)
export(SampleQuantiles)
export(cqv_versatile)
export(cv_versatile)
import(R6)
import(SciViews)
import(boot)
import(dplyr)
import(utils)
importFrom(MBESS,conf.limits.nct)
importFrom(stats,qchisq)
importFrom(stats,qnorm)
importFrom(stats,quantile)
importFrom(stats,sd)
